import QtQuick 1.1
import qb.components 1.0
import qb.base 1.0;
import "wastecollection.js" as WastecollectionJS

/// Application to manage retrieve the list of date for the waste bin collection
/// Actual display is handled via the ThermostatApp.qml and/or via the Tile

App {
	id: wastecollectionApp

// Start editing here
	property string wasteCollector : "cyclusnv.nl"
			//possible values for now: 	"mijnafvalwijzer.nl"
			//				"deafvalapp.nl"
			//				"cure-afvalbeheer.nl"
			//				"cyclusnv.nl")
	property string wasteZipcode : "1111AA"
	property string wasteHouseNr : "111"
//stop editing

	property url tileUrl : "WastecollectionTile.qml"
	property url thumbnailIcon: "./drawables/waste_menu.png"
	property string wasteDatesString
	property string wasteType
	property string wasteType2
	property string wasteIcon
	property bool wasteIconShow
	property string wasteIcon2
	property bool wasteIcon2Show
	property string wasteDateNext
	property string wasteDateNextType
	property string wasteDateNext2
	property string wasteDateNext2Type
	property string wasteDateNext3
	property string wasteDateNext3Type


	function init() {
		registry.registerWidget("tile", tileUrl, this, null, {thumbLabel: "Afvalkalender", thumbIcon: thumbnailIcon, thumbCategory: "general", thumbWeight: 30, baseTileWeight: 10, thumbIconVAlignment: "center"});
	}

	Component.onCompleted: {
		readWasteDates();
		datetimeTimerWaste.start();
	}


	function readWasteDates() {
		
		if (wasteCollector == "mijnafvalwijzer.nl") {
			readMijnafvalwijzer();
		}
		if (wasteCollector == "deafvalapp.nl") {   
			readDeafvalapp();
		}
		if (wasteCollector == "cure-afvalbeheer.nl") {   
			readCureAfvalbeheer();
		}
		if (wasteCollector == "cyclusnv.nl") {   
			readCureAfvalbeheer();		//only waste description differs from cure-afvalbeheer
		}
	}

	function wasteTypeMijnafvalwijzer(shortName) {
		switch (shortName) {
			case "gft": return 3;
			case "opk": return 2;
			case "pmd": return 1;
			case "restafval": return 0;
			case "takken": return 4;
			default: break;
		}
		return "?";
	}

	function wasteTypeCureAfvalbeheer(shortName) {
		switch (shortName) {
			case "Groent": return 3;	//cyclusnv plus cureafvalbeheer
			case "Papier": return 2;	//cureafvalbeheer
			case "Oud pa": return 2;	//cyclusnv.nl
			case "Plasti": return 1;	//cyclusnv.nl
			case "Restaf": return 0;	//cyclusnv plus cureafvalbeheer
			default: break;
		}
		return "?";
	}

	function wasteTypeCyclusnv(shortName) {
		switch (shortName) {
			case "Groent": return 3;
			case "Oud pa": return 2;
			case "Plasti": return 1;
			case "Restaf": return 0;
			default: break;
		}
		return "?";
	}

	function fullMonth(monthNum) {
		switch (monthNum) {
			case "01": return "januari";
			case "02": return "februari";
			case "03": return "maart";
			case "04": return "april";
			case "05": return "mei";
			case "06": return "juni";
			case "07": return "juli";
			case "08": return "augustus";
			case "09": return "september";
			case "10": return "oktober";
			case "11": return "november";
			case "12": return "december";
			default: break;
		}
		return "error_month";
	}

	function formatWasteDate(dateyymmdd) {
		return dateyymmdd.substring(8,10) + " " + fullMonth(dateyymmdd.substring(5,7))
	}

	function readCureAfvalbeheer() {
	
		var i = 0;
		var j = 0;
		wasteDatesString = "";
		var wasteType = "";
		var cureAfvalbeheerDates = [];

		var xmlhttp = new XMLHttpRequest();
		xmlhttp.onreadystatechange = function() {
			if (xmlhttp.readyState == XMLHttpRequest.DONE) {
				var aNode = xmlhttp.responseText;

				// read specific waste collection dates

				i = aNode.indexOf("DTSTART");
				if ( i > 0 ) {
					while (i > 0) {
						j = aNode.indexOf("SUMMARY", i);
						wasteType = wasteTypeCureAfvalbeheer(aNode.substring(j+8, j+14));
						cureAfvalbeheerDates.push(aNode.substring(i+8, i+12) + "-" + aNode.substring(i+12, i+14) + "-" + aNode.substring(i+14, i+16) + "," + wasteType);
						i = aNode.indexOf("DTSTART", i + 10);
					}
				}
				var tmp = WastecollectionJS.sortArray(cureAfvalbeheerDates);

				for (i = 0; i < tmp.length; i++) {
					wasteDatesString = wasteDatesString + tmp[i] + "\n";
				}
				writeWasteDates();
			}
		}
		xmlhttp.open("GET", "file:///root/waste/waste_calendar.ics", true);
		xmlhttp.send();
	}

	function readMijnafvalwijzer() {
	
		var i = 0;
		var j = 0;
		wasteDatesString = "";
		var wasteType = "";


		var xmlhttp = new XMLHttpRequest();
		xmlhttp.onreadystatechange=function() {
			if (xmlhttp.readyState == 4) {
				if (xmlhttp.status == 200) {
					var aNode = xmlhttp.responseText;

					// read specific waste collection dates

					i = aNode.indexOf("nameType");
					if ( i > 0 ) {
						while (i > 0) {
	 						aNode = aNode.slice(i);
							j = aNode.indexOf('"', 12);
							wasteType = wasteTypeMijnafvalwijzer(aNode.substring(11, j));
							i = aNode.indexOf("date");
							wasteDatesString = wasteDatesString + aNode.substring(i+7, i+17) + "," + wasteType + "\n";
							i = aNode.indexOf("nameType", 10);
						}
					}
					writeWasteDates();
				}
			}
		}
		xmlhttp.open("GET", "http://json.mijnafvalwijzer.nl/?method=postcodecheck&postcode=" + wasteZipcode + "&street=&huisnummer=" + wasteHouseNr + "&toevoeging=", true);
		xmlhttp.send();
	}


	function readDeafvalapp() {
	
		var indexGFT = 0;
		var indexPAPIER = 0;
		var indexREST = 0;
		var indexPLASTIC = 0;

		var i = 0;
		var j = 0;
		wasteDatesString = "";
		var wasteType = "";
		var fileDate = "";
		var lastGFTfileDate = "";
		var lastPAPIERfileDate = "";
		var lastPLASTICfileDate = "";
		var lastRESTfileDate = "";
		var deafvalappDates = [];

		var xmlhttp = new XMLHttpRequest();
		xmlhttp.onreadystatechange=function() {
			if (xmlhttp.readyState == 4) {
				if (xmlhttp.status == 200) {
					var aNode = xmlhttp.responseText;

					// read start index of all wastetypes

					indexGFT = aNode.indexOf("GFT");
					indexPAPIER = aNode.indexOf("PAPIER");
					indexPLASTIC = aNode.indexOf("PLASTIC");
					indexREST = aNode.indexOf("REST");

					// find first dates in the future of each type - GFT
 
					i = aNode.indexOf(';', indexGFT);
					while (i < indexPAPIER) {
						j = aNode.indexOf(';', i+1);

						if (j < indexPAPIER) {
							fileDate = aNode.substring(i+7, j) + "-" + aNode.substring(i+4, i+6) + "-" + aNode.substring(i+1, i+3);
							deafvalappDates.push(fileDate + "_3")
						}
						i = j;
					}
					
					// find first dates in the future of each type - PAPIER

					i = aNode.indexOf(';', indexPAPIER);
					while (i < indexPLASTIC) {
						j = aNode.indexOf(';', i+1);

						if (j < indexPLASTIC) {
							fileDate = aNode.substring(i+7, j) + "-" + aNode.substring(i+4, i+6) + "-" + aNode.substring(i+1, i+3);
							deafvalappDates.push(fileDate + "_2")
						}
						i = j;
					}
					
					// find first dates in the future of each type - PLASTIC

					i = aNode.indexOf(';', indexPLASTIC);
					while (i < indexREST) {
						j = aNode.indexOf(';', i+1);

						if (j < indexREST) {
							fileDate = aNode.substring(i+7, j) + "-" + aNode.substring(i+4, i+6) + "-" + aNode.substring(i+1, i+3);
							deafvalappDates.push(fileDate + "_1")
						}
						i = j;
					}
					
					// find first dates in the future of each type - REST

					i = aNode.indexOf(';', indexREST);
					while (i < (aNode.length -3)) {
						j = aNode.indexOf(';', i+1);

						if (j > 0) {
							fileDate = aNode.substring(i+7, j) + "-" + aNode.substring(i+4, i+6) + "-" + aNode.substring(i+1, i+3);
							deafvalappDates.push(fileDate + "_0")
						}
						i = j;
					}

					// sort dates

					var tmp = WastecollectionJS.sortArray(deafvalappDates);

					for (i = 0; i < tmp.length; i++) {
						wasteDatesString = wasteDatesString + tmp[i] + "\n";
					}
					writeWasteDates();
				}
			}
		}
		xmlhttp.open("GET", "http://dataservice.deafvalapp.nl/dataservice/DataServiceServlet?service=OPHAALSCHEMA&land=NL&postcode=" + wasteZipcode + "&huisnr=" + wasteHouseNr + "&huisnrtoev=", true);
		xmlhttp.send();
	}

	function writeWasteDates() {
   		var doc2 = new XMLHttpRequest();
   		doc2.open("PUT", "file:///var/volatile/tmp/wasteDates.txt");
   		doc2.send(wasteDatesString);
	}

	// write icondata to tmp file to for use in Thermostat App (optional)

	function writeWasteIconData() {
   		var doc3 = new XMLHttpRequest();
   		doc3.open("PUT", "file:///var/volatile/tmp/wasteDateIcon.txt");
   		doc3.send(wasteIcon);
	}

	function writeWasteIconData2() {
   		var doc4 = new XMLHttpRequest();
   		doc4.open("PUT", "file:///var/volatile/tmp/wasteDateIcon2.txt");
   		doc4.send(wasteIcon2);
	}

	function wasteTypeFriendlyName(wasteCode) {
		switch (wasteCode) {
			case "3" : return "Groente/Fruit/Tuinafval";
			case "2" : return "Papier/karton";
			case "1" : return "Plastic/Metaal/Drankpakken";
			case "0" : return "Restafval";
			case "4" : return "Snoeiafval";
			default: break;
		}
		return "Onbekend";
	}
	
	function wasteIconURL(wasteCode) {
		switch (wasteCode) {
			case "3" : return "../wastecollection/drawables/gftDim.png";
			case "2" : return "../wastecollection/drawables/papierDim.png";
			case "1" : return "../wastecollection/drawables/pmdDim.png";
			case "0" : return "../wastecollection/drawables/restafvalDim.png";
			case "4" : return "../wastecollection/drawables/snoeiafvalDim.png";
			default: break;
		}
		return "";
	}

	function readCollectedWasteDates(searchStr) {

		var fileWasteDate = "";
		wasteIcon = "";
		wasteIcon2 = "";
		writeWasteIconData();
		writeWasteIconData2();
		wasteIconShow = false;
		wasteIcon2Show = false;
		wasteDateNext = "";
		wasteDateNextType = "";
		wasteDateNext2 = "";
		wasteDateNext2Type = "";
		wasteDateNext3 = "";
		wasteDateNext3Type = "";

		var counter = 0;
		var i = 0;
		var wastefile= new XMLHttpRequest();
		wastefile.onreadystatechange = function() {
			if (wastefile.readyState == XMLHttpRequest.DONE) {
				var response = wastefile.responseText;
				for (i = 0; i < response.length/13; i++) {
					fileWasteDate = response.slice(i*13,i*13+10);

					if (counter == 2) {
						counter = 3; 
						wasteDateNext3Type = wasteTypeFriendlyName(response.slice(i*13+11,i*13+12));
						wasteDateNext3 = formatWasteDate(response.slice(i*13,i*13+10));
					}
					if (counter == 1) {
						counter = 2;
						wasteDateNext2Type = wasteTypeFriendlyName(response.slice(i*13+11,i*13+12));
						wasteDateNext2 = formatWasteDate(response.slice(i*13,i*13+10));
						if (searchStr.localeCompare(fileWasteDate) == 0) {
							wasteIcon2Show = true;
							wasteIconShow = false;
							wasteIcon2 = wasteIconURL(response.slice(i*13+11,i*13+12));
							writeWasteIconData2();
						}
					}
					if (counter == 0) {
						if (searchStr.localeCompare(fileWasteDate) <= 0) {
							counter = 1;
							wasteDateNextType = wasteTypeFriendlyName(response.slice(i*13+11,i*13+12));
							wasteDateNext = formatWasteDate(response.slice(i*13,i*13+10));
							if (searchStr.localeCompare(fileWasteDate) == 0) {
								wasteIconShow = true;
								wasteIcon = wasteIconURL(response.slice(i*13+11,i*13+12));
								writeWasteIconData();
							}
						}
					}
				}
			}
		}
		wastefile.open("GET", "file:///var/volatile/tmp/wasteDates.txt", true);
		wastefile.send();
	}

	/// calculates miliseconds till next 18:00 from now
	function getMSecTill6oclock() {
		var now = new Date();
		var nowUtc = Date.UTC(now.getFullYear(), now.getMonth(), now.getDate(), now.getHours(), now.getMinutes(), now.getSeconds(),now.getMilliseconds());
		var addaday = 0;
		if (now.getHours() >= 18) {
			addaday = 1;
		}
		var sixOclock = Date.UTC(now.getFullYear(), now.getMonth(), now.getDate() + addaday, 18, 0, 0, 0);
		return sixOclock - nowUtc;
	}

	function updateWasteIcon() {

		var now = new Date( (new Date)*1 + 1000*3600*7); //current date/time plus 7 hours to show icon from 17:00 the day before onwards
		var strMon = now.getMonth() + 1;
		if (strMon < 10) {
			strMon = "0" + strMon;
		}
		var strDay = now.getDate();
		if (strDay < 10) {
			strDay = "0" + strDay;
		}
		var searchDateStr = now.getFullYear() + "-" + strMon + "-" +  strDay;
		readCollectedWasteDates(searchDateStr);
		writeWasteIconData();
	}

	Timer {
		id: datetimeTimerWaste
		repeat: true
		running: false
		interval: 20000	//wait 20 sec after reboot before for triggering, after that only at 18:00 daily

		onTriggered: {
			updateWasteIcon();	//update data on tile
			readWasteDates();	//get new calender update for display from next day onwards (although refresh once a day is a bit often maybe)
			interval = getMSecTill6oclock();
		}
	}
}
